@@ -1,7 +1,7 @@
 package Bencher;
 
-our $DATE = '2015-11-29'; # DATE
-our $VERSION = '0.15'; # VERSION
+our $DATE = '2015-12-09'; # DATE
+our $VERSION = '0.16'; # VERSION
 
 use 5.010001;
 use strict;
@@ -567,6 +567,7 @@ sub _gen_items {
             seq  => $i,
             name => $item_name,
             code => $code,
+            _permute => $h,
         };
     } # ITER
 
@@ -878,6 +879,7 @@ _
                            list-participant-modules
                            list-datasets
                            list-items
+                           show-items-results
                            bench
                        /]
                     # list-functions
@@ -920,6 +922,11 @@ _
                     summary => 'Shortcut for -a list-items',
                     code => sub { $_[0]{action} = 'list-items' },
                 },
+                show_items_results => {
+                    is_flag => 1,
+                    summary => 'Shortcut for -a show-items-results',
+                    code => sub { $_[0]{action} = 'show-items-results' },
+                },
             },
             tags => ['category:action'],
         },
@@ -1263,45 +1270,89 @@ sub bencher {
         return [200, "OK", \@res, \%resmeta];
     }
 
-    if ($action eq 'bench') {
+    if ($action =~ /\A(show-items-results|bench)\z/) {
+        require Module::Load;
+
+        my $participants = $parsed->{participants};
+        my $envres = [200, "OK", [], {}];
+
         my $return_resmeta =
             $args{-cmdline_r} && (($args{-cmdline_r}{format} // '') !~ /json/) ?
             0 : 1;
 
-        require Benchmark::Dumb;
-        require Devel::Platform::Info if $return_resmeta;
-        require Module::Load;
-        require Sys::Load if $return_resmeta;
+        $envres->[3]{'func.module_versions'}{perl} = $^V if $return_resmeta;
 
-        my $envres = [200, "OK", [], {}];
+        my $code_load = sub {
+            no strict 'refs';
+            my $mod = shift;
+            $log->tracef("Loading module: %s", $mod);
+            Module::Load::load($mod);
+            if ($return_resmeta) {
+                # we'll just use ${"$mod\::VERSION"} because we are already
+                # loading the module
+                $envres->[3]{'func.module_versions'}{$mod} =
+                    ${"$mod\::VERSION"};
+            }
+        };
+
+        $code_load->('Benchmark::Dumb');
+        $code_load->('Devel::Platform::Info') if $return_resmeta;
+        $code_load->('Sys::Info')             if $return_resmeta;
+        $code_load->('Sys::Load')             if $return_resmeta;
 
         # load all modules
         {
             my %seen;
             my @modules = _get_participant_modules($parsed);
             for my $mod (@modules) {
-                $log->tracef("Loading module: %s", $mod);
-                Module::Load::load($mod);
+                $code_load->($mod);
             }
         }
 
         my $on_failure = $args{on_failure} // $parsed->{on_failure} // 'die';
-        if ($on_failure eq 'skip') {
+        {
             my $fitems = [];
             for my $it (@$items) {
                 $log->tracef("Testing code for item #%d (%s) ...",
                              $it->{seq}, $it->{name});
-                eval { $it->{code}->() };
-                if ($@) {
-                    warn "Skipping item #$it->{seq} ($it->{name}) ".
-                        "due to failure: $@\n";
-                    next;
+                eval {
+                    my $result_is_list = $participants->[
+                        $it->{_permute}{participant} ]{result_is_list} // 0;
+                    $it->{_result} = $result_is_list ?
+                        [$it->{code}->()] : $it->{code}->();
+                };
+                my $err = $@;
+                if ($err) {
+                    if ($on_failure eq 'skip' || $action eq 'show-items-results') {
+                        warn "Skipping item #$it->{seq} ($it->{name}) ".
+                            "due to failure: $err\n";
+                        next;
+                    } else {
+                        die "Item #$it->{seq} ($it->{name}) fails: $err\n";
+                    }
                 }
                 push @$fitems, $it;
             }
             $items = $fitems;
         }
 
+        if ($action eq 'show-items-results') {
+            if ($return_resmeta) {
+                $envres->[2] = [map {$_->{_result}} @$items];
+            } else {
+                require Data::Dump;
+                $envres->[2] = join(
+                    "",
+                    map {(
+                        "#$_->{seq} ($_->{name}):\n",
+                        Data::Dump::dump($_->{_result}),
+                        "\n\n",
+                    )} @$items
+                );
+            }
+            goto RETURN_RESULT;
+        }
+
         if ($return_resmeta) {
             $envres->[3]{'func.sysload_before'} = [Sys::Load::getload()];
             $envres->[3]{'_time_start'} = time();
@@ -1315,6 +1366,13 @@ sub bencher {
             "silent",
         );
 
+        if ($return_resmeta) {
+            $envres->[3]{'func.elapsed_time'} =
+                time() - $envres->[3]{'_time_start'};
+            delete $envres->[3]{'_time_start'};
+            $envres->[3]{'func.sysload_after'} = [Sys::Load::getload()];
+        }
+
         for my $seq (sort {$a<=>$b} keys %$tres) {
             my $it = _find_record_by_seq($items, $seq);
             push @{$envres->[2]}, {
@@ -1383,14 +1441,13 @@ sub bencher {
         } # FORMAT
 
         if ($return_resmeta) {
-            $envres->[3]{'func.sysload_after'} = [Sys::Load::getload()];
             $envres->[3]{'func.platform_info'} =
                 Devel::Platform::Info->new->get_info;
-            $envres->[3]{'func.elapsed_time'} =
-                time() - $envres->[3]{'_time_start'};
-            delete $envres->[3]{'_time_start'};
+            my $info = Sys::Info->new;
+            $envres->[3]{'func.cpu_info'} = [$info->device('CPU')->identify];
         }
 
+      RETURN_RESULT:
         return $envres;
     }
 
@@ -1428,7 +1485,7 @@ Bencher - A benchmark framework
 
 =head1 VERSION
 
-This document describes version 0.15 of Bencher (from Perl distribution Bencher), released on 2015-11-29.
+This document describes version 0.16 of Bencher (from Perl distribution Bencher), released on 2015-12-09.
 
 =head1 SYNOPSIS
 
